import numpy as np
import pandas as pd
import talib


class Strategy:

    def __init__(self):
        # Parameters
        self.point_value = 0.00001
        self.bb_period = 55
        self.dev_period = 1.4
        self.max_space = 900

    def execute(self, df):
        """
        Requires 9 parameters
        : param df:
        """
        data = df.copy()
        data.index = pd.to_datetime(data.index, unit="s", utc=True)

        data["returns"] = np.log(data.close.div(data.close.shift(1)))
        data["direction"] = np.sign(data["returns"])
        data["hour"] = data.index.hour

        data["bb_upper"], data["bb_middle"], data["bb_lower"] = talib.BBANDS(data["close"], timeperiod=self.bb_period,
                                                                             nbdevup=self.dev_period, nbdevdn=self.dev_period,
                                                                             matype=0)
        data["lower_space"] = abs(data["bb_middle"] - data["bb_lower"])
        data["upper_space"] = abs(data["bb_upper"] - data["bb_middle"])
        data["total_space"] = abs(data["bb_upper"] - data["bb_lower"])
        data["total_to_half"] = data["total_space"] - data["lower_space"]
        data["expansion_rate"] = data["total_space"] / data["total_space"].shift(1)
        data["distance"] = data["close"] - data["bb_middle"]

        data.dropna(inplace=True)

        # TRADING CONDITIONS

        # Neutral Condition
        condc = data["total_space"] > self.max_space * self.point_value

        # Sell Condition
        cond1 = data["high"] > data["bb_upper"]

        # Buy Condition
        cond11 = data["low"] < data["bb_lower"]

        data["position"] = np.where(condc & cond1, -1, np.nan)
        data["position"] = np.where(condc & cond11, 1, data["position"])
        data["position"] = np.where(data["distance"] * data["distance"].shift(1) < 0, 0, data["position"])
        data.position = data.position.ffill().fillna(0)

        self.prepared_data = data.copy()
